from uuid import UUID

from pydantic import BaseModel, Field, EmailStr


class LoginRequest(BaseModel):
    username: str = Field(..., description="User Name")
    password: str = Field(..., description="Password")


class SessionToken(BaseModel):
    access_token: str = Field(..., description="Access Token")
    refresh_token: str = Field(..., description="Refresh Token")

class LoginResponse(BaseModel):
    current_time: str = Field(..., description="Login Time")
    region: str = Field(..., description="User Resion")
    tokens: SessionToken = Field(default_factory=SessionToken, description="Session Token")


class RefreshTokenRequest(BaseModel):
    access_token: str = Field(..., description="Access Token")
    refresh_token: str = Field(..., description="Refresh Token")


class RefreshTokenResponse(BaseModel):
    access_token: str = Field(..., description="Access Token")
    refresh_token: str = Field(..., description="Refresh Token")


class CreditResponse(BaseModel):
    free: int = Field(..., description="Free Credit")
    reward: int = Field(..., description="Reward Credit")
    contract: int = Field(..., description="Contract Credit")
    paid: int = Field(..., description="Paid Credit")
    total: int = Field(..., description="Total Credit")


class Authorities(BaseModel):
    compressor: bool = Field(False, description="Compressor Authority")
    searcher: bool = Field(False, description="Searcher Authority")
    launcher: bool = Field(False, description="Launcher Authority")

class UserDetailResponse(BaseModel):
    first_name: str = Field(..., description="First Name")
    last_name: str = Field(..., description="Last Name")
    company: str = Field(..., description="Company")

class UserResponse(BaseModel):
    user_id: str = Field(..., description="User ID")
    email: EmailStr = Field(..., description="Email")
    username: str = Field(..., description="User Name")
    detail_data: UserDetailResponse = Field(..., description="User Detail")
    # credit: int = Field(..., description="Credit")
    # is_active: bool = Field(..., description="Active Status")
    # is_admin: bool = Field(..., description="Admin Status")
    # privacy_policy_agreement: bool = Field(..., description="Privacy Policy Agreement")
    # marketing_agreement: bool = Field(..., description="Marketing Agreement")
    # current_time: str = Field(..., description="Current Time")
    # created_time: str = Field(..., description="Created Time")
    # last_login_time: str = Field(..., description="Last Login Time")
    # authorities: Authorities = Field(default_factory=Authorities, description="NP Module Authorities")



import requests, json
from functools import wraps
from loguru import logger

from netspresso.schemas.auth import LoginRequest, RefreshTokenRequest, LoginResponse, RefreshTokenResponse, UserResponse
from netspresso.client.utils.common import get_headers
from netspresso.client.utils.token import check_jwt_exp
from netspresso.client.config import Config, EndPoint

def validate_token(func) -> None:
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        if not check_jwt_exp(self.user_session.access_token):
            self.user_session.__reissue_token()
        return func(self, *args, **kwargs)

    return wrapper

class SessionClient:
    def __init__(self, email: str, password: str, config: Config = None):
        """Initialize the UserSession.

        Args:
            email (str): The email address for a user account.
            password (str): The password for a user account.
        """

        self.email = email
        self.password = password
        self.config = config if config is not None else Config(EndPoint.GENRAL)
        self.host = self.config.HOST
        self.port = self.config.PORT
        self.uri_prefix = self.config.URI_PREFIX
        self.base_url = f"{self.host}:{self.port}{self.uri_prefix}"
        self.user_id = None
        self.__login()
        self.__get_user_info()

    def __login(self) -> None:
        try:
            url = f"{self.base_url}/auth/local/login"
            data = LoginRequest(username=self.email, password=self.password)
            response = requests.post(url, json=data.dict(), headers=get_headers())
            response_body = json.loads(response.text)

            if response.status_code == 200 or response.status_code == 201:
                session = LoginResponse(**response_body)
                self.access_token = session.tokens.access_token
                self.refresh_token = session.tokens.refresh_token
                logger.info("Login successfully")
            else:
                raise Exception(response_body["detail"])

        except Exception as e:
            logger.error(f"Login failed. Error: {e}")
            raise e

    def __get_user_info(self):
        try:
            url = f"{self.base_url}/user"
            response = requests.get(url, headers=get_headers(access_token=self.access_token))
            response_body = json.loads(response.text)

            if response.status_code == 200 or response.status_code == 201:
                user_info = UserResponse(**response_body)
                self.user_id = user_info.user_id
                logger.info("successfully got user information")
            else:
                raise Exception(response_body["detail"])

        except Exception as e:
            logger.error(f"Failed to get user information. Error: {e}")
            raise e

    def __reissue_token(self) -> None:
        try:
            url = f"{self.base_url}/token"
            data = RefreshTokenRequest(access_token=self.access_token, refresh_token=self.refresh_token)
            response = requests.post(url, data=data.json(), headers=get_headers(json_type=True))
            response_body = json.loads(response.text)

            if response.status_code == 200 or response.status_code == 201:
                session = RefreshTokenResponse(**response_body)
                self.access_token = session.access_token
                self.refresh_token = session.refresh_token
            else:
                raise Exception(response_body["detail"])

        except Exception as e:
            raise e

class BaseClient:
    user_session: SessionClient = None
    def __init__(self, email=None, password=None, user_session=None):
        """Initialize the Model Compressor.

        Args:
            email (str): The email address for a user account.
            password (str): The password for a user account.
            user_session (SessionClient): The SessionClient object.

        Available constructors: 
            BaseClient(email='USER_EMAIL',password='PASSWORD')
            BaseClient(user_session=SessionClient(email='USER_EMAIL',password='PASSWORD')
        """

        if user_session:
            # Case 1: Creating from a user_session
            self.user_session = user_session
        elif email and password:
            # Case 2: Creating from email and password
            self.user_session = SessionClient(email=email, password=password)
        else:
            raise NotImplementedError("There is no avaliable constructors for given paremeters.")


    @validate_token
    def get_credit(self) -> int:
        """Get the available NetsPresso credits.

        Raises:
            e: If an error occurs while getting credit information.

        Returns:
            int: The total amount of available NetsPresso credits.
        """

        try:
            credit = self.client.get_credit(access_token=self.access_token)
            logger.info(f"Get Credit successfully. Credit: {credit.total}")

            return credit.total

        except Exception as e:
            logger.error(f"Get Credit failed. Error: {e}")
            raise e


from dataclasses import dataclass, field
from typing import Any, List

from netspresso.compressor.client.utils.enum import Policy, LayerNorm, GroupPolicy


@dataclass
class AvailableLayer:
    """Represents an available layer for compression.

    Attributes:
        name (str): The name of the layer.
        values (List[Any]): The compression parameters for the layer.
        use (bool): The compression selection status for the layer.
        channels (List[int]): The channel information for the layer.
    """

    name: str
    values: List[Any] = field(default_factory=list)
    use: bool = field(default=False, repr=False)
    channels: List[int] = field(default_factory=list)


@dataclass
class Options:
    """Represents an options for compression.

    Attributes:
        reshape_channel_axis (int): 
        policy (str): 
        layer_norm (str): 
        group_policy (str): 
    """

    reshape_channel_axis: int = -1
    policy: Policy = Policy.AVERAGE
    layer_norm: LayerNorm = LayerNorm.STANDARD_SCORE
    group_policy: GroupPolicy = GroupPolicy.AVERAGE


@dataclass
class CompressionInfo:
    """Represents compression information for a model.

    Attributes:
        compressed_model_id (str): The ID of the compressed model.
        compression_id (str): The ID of the compression.
        compression_method (str): The compression method used.
        available_layers (List[AvailableLayer]): The compressible layers information.

            AvailableLayer Attributes:
                - name (str): The name of the layer.
                - values (List[Any]): The compression parameters for the layer.
                - channels (List[int]): The channel information for the layer.
        
        options(Options, optional): The options for pruning method.

        original_model_id (str): The ID of the original model.
    """

    compressed_model_id: str = ""
    compression_id: str = ""
    compression_method: str = ""
    available_layers: List[AvailableLayer] = field(default_factory=list)
    original_model_id: str = ""
    options: Options = field(default_factory=Options)

    def set_available_layers(self, available_layers):
        self.available_layers = [AvailableLayer(**available_layer.dict()) for available_layer in available_layers]



from typing import Dict, List, Union
from loguru import logger
from urllib import request
import time

from netspresso.client import BaseClient, validate_token
from netspresso.launcher.schemas import (
    LauncherFunction,
    ModelFramework,
    TaskStatus,
    DataType,
    SoftwareVersion,
    DeviceName,
    JETSON_DEVICES,
    HardwareType,
    ONLY_INT8_DEVICES,
)
from netspresso.launcher.schemas.model import Model, ConversionTask, BenchmarkTask, InputShape, TargetDevice
from netspresso.launcher.utils.devices import (
    filter_devices_with_device_name,
    filter_devices_with_device_software_version,
    filter_devices_with_hardware_type,
)
from netspresso.launcher.client import LauncherAPIClient


class Launcher(BaseClient):
    target_function: LauncherFunction = LauncherFunction.GENERAL

    def __init__(self, email=None, password=None, user_session=None):
        """Initialize the Model Compressor.

        Args:
            email (str): The email address for a user account.
            password (str): The password for a user account.
            user_session (SessionClient): The SessionClient object.

        Available constructors:
            Launcher(email='USER_EMAIL',password='PASSWORD')
            Launcher(user_session=SessionClient(email='USER_EMAIL',password='PASSWORD')
        """
        super().__init__(email=email, password=password, user_session=user_session)
        self.client = LauncherAPIClient(user_sessoin=self.user_session)

    @validate_token
    def upload_model(self, model_file_path: str) -> Model:
        """Upload a model for launcher.

        Args:
            model_file_path (str): The file path of the model.

        Raises:
            e: If an error occurs while uploading the model.

        Returns:
            Model: Uploaded launcher model object.
        """
        return self.client.upload_model(model_file_path=model_file_path, target_function=self.__class__.target_function)


class ModelConverter(Launcher):
    target_function: LauncherFunction = LauncherFunction.CONVERT

    @validate_token
    def convert_model(
        self,
        model: Union[str, Model],
        input_shape: InputShape,
        target_framework: Union[str, ModelFramework],
        data_type: DataType = DataType.FP16,
        target_device: TargetDevice = None,
        wait_until_done: bool = True,
        target_device_name: DeviceName = None,
        target_software_version: SoftwareVersion = None,
    ) -> ConversionTask:
        """Convert a model into the type the specific framework.

        Args:
            model (str): The uuid of the model or Launcher Model Object.
            input_shape (InputShape) : target input shape to convert. (ex: dynamic batch to static batch)
            target_framework (ModelFramework | str): the target framework name.
            data_type (DataType): data type of the model.
            target_device (TargetDevice): target device. If it's not set, target_device_name and target_software_version have to be set.
            wait_until_done (bool): if true, wait for the conversion result before returning the function. If false, request the conversion and return the function immediately.
            target_device_name (DeviceName): target device name. Necessary field if target_device is not given.
            target_software_version (SoftwareVersion): target_software_version. Necessary field if target_device_name is one of jetson devices.

        Raises:
            e: If an error occurs while converting the model.

        Returns:
            ConversionTask: model conversion task object.
        """
        model_uuid = model
        if type(model) is Model:
            model_uuid = model.model_uuid
            if input_shape is None and model.input_shape is not None:
                input_shape = model.input_shape
            if target_framework is None and model.framework is not None:
                target_framework = model.framework

        if target_device is None:
            if target_device_name is None:
                raise NotImplementedError(
                    "The conversion is unavailable. Please set target_device or target_device_name."
                )

            elif target_device_name in JETSON_DEVICES and target_software_version is None:
                raise NotImplementedError(
                    "The conversion is unavailable. Please set JetPack version with target_software_version for Jetson Devices."
                )

            if type(model) is not Model:
                raise NotImplementedError(
                    "The conversion is unavailable. Please set target_device while using model's uuid string for the conversion."
                )

            # Check available int8 converting devices
            if data_type == DataType.INT8:
                if target_device_name not in ONLY_INT8_DEVICES:
                    raise Exception(f"int8 converting supports only {ONLY_INT8_DEVICES}.")
            else:  # FP16, FP32
                if target_device_name in ONLY_INT8_DEVICES:
                    raise Exception(f"{ONLY_INT8_DEVICES} only support int8 data types.")

            devices = filter_devices_with_device_name(name=target_device_name, devices=model.available_devices)

            if target_device_name in JETSON_DEVICES:
                devices = filter_devices_with_device_software_version(
                    software_version=target_software_version, devices=devices
                )

            if len(devices) < 1:
                raise NotImplementedError(
                    "The conversion is unavailable. There is no available device with given target_device_name and target_software_version."
                )

            target_device = devices[0]

        logger.info(f"Converting Model for {target_device.device_name} ({target_framework})")

        conversion_task = self.client.convert_model(
            model_uuid=model_uuid,
            input_shape=input_shape,
            target_framework=target_framework,
            target_device=target_device.device_name,
            data_type=data_type,
            software_version=target_device.software_version,
        )

        conversion_task = self.get_conversion_task(conversion_task)

        if wait_until_done:
            while conversion_task.status in [TaskStatus.IN_QUEUE, TaskStatus.IN_PROGRESS]:
                conversion_task = self.get_conversion_task(conversion_task)
                time.sleep(1)

        return conversion_task

    @validate_token
    def get_conversion_task(self, conversion_task: Union[str, ConversionTask]) -> ConversionTask:
        """Get the conversion task information with given conversion task or conversion task uuid.

        Args:
            conversion_task (ConversionTask | str): Launcher Model Object or the uuid of the conversion task.

        Raises:
            e: If an error occurs while getting the conversion task information.

        Returns:
            ConversionTask: model conversion task object.
        """
        conversion_task_uuid = None
        if type(conversion_task) is str:
            conversion_task_uuid = conversion_task
        elif type(conversion_task) is ConversionTask:
            conversion_task_uuid = conversion_task.convert_task_uuid
        else:
            raise NotImplementedError(
                "There is no available function for the given parameter. The 'conversion_task' should be a UUID string or a ModelConversion object."
            )
        return self.client.get_conversion_task(conversion_task_uuid=conversion_task_uuid)

    @validate_token
    def download_converted_model(self, conversion_task: Union[str, ConversionTask], dst: str):
        """Download the converted model with given conversion task or conversion task uuid.

        Args:
            conversion_task (ConversionTask | str): Launcher Model Object or the uuid of the conversion task.

        Raises:
            e: If an error occurs while getting the conversion task information.

        Returns:
            ConversionTask: model conversion task object.
        """
        conversion_task_uuid = None
        if type(conversion_task) is str:
            conversion_task_uuid = conversion_task
        elif type(conversion_task) is ConversionTask:
            conversion_task_uuid = conversion_task.convert_task_uuid

        conversion_result: ConversionTask = self.get_conversion_task(conversion_task_uuid)
        if conversion_result.status is TaskStatus.ERROR:
            raise FileNotFoundError("The conversion is Failed. There is no file available for download.")
        if conversion_result.status is not TaskStatus.FINISHED:
            raise FileNotFoundError(
                "The conversion is in progress. There is no file available for download at the moment."
            )

        download_url = self.client.get_converted_model(conversion_task_uuid=conversion_result.convert_task_uuid)
        request.urlretrieve(download_url, dst)
        logger.info(f"The file has been successfully downloaded at : {dst}")


class ModelBenchmarker(Launcher):
    target_function: LauncherFunction = LauncherFunction.BENCHMARK

    @validate_token
    def benchmark_model(
        self,
        model: Union[ConversionTask, Model, str],
        target_device: TargetDevice = None,
        data_type: DataType = DataType.FP16,
        wait_until_done: bool = True,
        target_device_name: DeviceName = None,
        target_software_version: SoftwareVersion = None,
        hardware_type: HardwareType = None,
    ) -> BenchmarkTask:
        """Benchmark given model on the specified device.

        Args:
            model (ConversionTask | Model | str): The conversion task object, Launcher Model Object or the uuid of the model.
            target_device (TargetDevice): target device. If it's not set, target_device_name and target_software_version have to be set.
            data_type (DataType): data type of the model.
            wait_until_done (bool): if true, wait for the conversion result before returning the function. If false, request the conversion and return the function immediately.
            target_device_name (DeviceName): target device name. Necessary field if target_device is not given.
            target_software_version (SoftwareVersion): target_software_version. Necessary field if target_device_name is one of jetson devices.
            hardware_type (HardwareType): hardware_type. Acceleration options for the processor to the model inference.

        Raises:
            e: If an error occurs while benchmarking of the model.

        Returns:
            BenchmarkTask: model benchmark task object.
        """
        model_uuid = None
        benchmark_data_type = data_type

        if target_device is None and type(model) is not ConversionTask:
            if target_device_name is None:
                raise NotImplementedError(
                    "The benchmark is unavailable. Please set target_device or target_device_name."
                )

            elif target_device_name in JETSON_DEVICES and target_software_version is None:
                raise NotImplementedError(
                    "The benchmark is unavailable. Please set JetPack version with target_software_version for Jetson Devices."
                )

            if type(model) is not Model:
                raise NotImplementedError(
                    "The benchmark is unavailable. Please set target_device while using model's uuid string for the conversion."
                )

            # Check available int8 converting devices
            if data_type == DataType.INT8:
                if target_device_name not in ONLY_INT8_DEVICES:
                    raise Exception(f"int8 converting supports only {ONLY_INT8_DEVICES}.")
            else:  # FP16, FP32
                if target_device_name in ONLY_INT8_DEVICES:
                    raise Exception(f"{ONLY_INT8_DEVICES} only support int8 data types.")

            if hardware_type == HardwareType.HELIUM and target_device_name not in ONLY_INT8_DEVICES:
                raise Exception(f"{ONLY_INT8_DEVICES} only support helium hardware type.")

            devices = filter_devices_with_device_name(name=target_device_name, devices=model.available_devices)

            if target_device_name in JETSON_DEVICES:
                devices = filter_devices_with_device_software_version(
                    software_version=target_software_version, devices=devices
                )

            devices = filter_devices_with_hardware_type(hardware_type=hardware_type, devices=devices)

            if len(devices) < 1:
                raise NotImplementedError(
                    "The benchmark is unavailable. There is no available device with given target_device_name and target_software_version."
                )

            target_device = devices[0]

        target_device_name = target_device.device_name if target_device is not None else None
        target_software_version = target_device.software_version if target_device is not None else None
        target_hardware_type = target_device.hardware_type if target_device is not None else None

        if type(model) is str:
            model_uuid = model
        elif type(model) is Model:
            model_uuid = model.model_uuid
        elif type(model) is ConversionTask:
            model_uuid = model.output_model_uuid
            if target_device_name is None:
                target_device_name = model.target_device_name
            if target_software_version is None:
                target_software_version = model.software_version
            if target_hardware_type is None:
                target_hardware_type = model.hardware_type

        if target_device_name is None:
            raise NotImplementedError(
                "There is no avaliable function for given paremeter. Please specify the target device."
            )

        model_benchmark: BenchmarkTask = self.client.benchmark_model(
            model_uuid=model_uuid,
            target_device=target_device_name,
            data_type=benchmark_data_type,
            software_version=target_software_version,
            hardware_type=target_hardware_type,
        )
        model_benchmark = self.get_benchmark_task(benchmark_task=model_benchmark)

        if wait_until_done:
            while model_benchmark.status in [TaskStatus.IN_QUEUE, TaskStatus.IN_PROGRESS]:
                model_benchmark = self.get_benchmark_task(benchmark_task=model_benchmark)
                time.sleep(1)

        return model_benchmark

    @validate_token
    def get_benchmark_task(self, benchmark_task: Union[str, BenchmarkTask]) -> BenchmarkTask:
        """Get the benchmark task information with given benchmark task or benchmark task uuid.

        Args:
            benchmark_task (BenchmarkTask | str): Launcher Benchmark Object or the uuid of the benchmark task.

        Raises:
            e: If an error occurs while getting the benchmark task information.

        Returns:
            BenchmarkTask: model benchmark task object.
        """
        task_uuid = None
        if type(benchmark_task) is str:
            task_uuid = benchmark_task
        elif type(benchmark_task) is BenchmarkTask:
            task_uuid = benchmark_task.benchmark_task_uuid
        else:
            raise NotImplementedError(
                "There is no available function for the given parameter. The 'benchmark_task' should be a UUID string or a ModelBenchmark object."
            )

        return self.client.get_benchmark(benchmark_task_uuid=task_uuid)



from typing import List
from netspresso.launcher.schemas import DeviceName
from netspresso.launcher.schemas.model import TargetDevice


def filter_devices_with_device_name(name: DeviceName, devices: List[TargetDevice]) -> List[TargetDevice]:
    """Filter the devices with given device name.

    Args:
        name (DeviceName): The device name.
        devices (List[TargetDevice]): The list of devices
    Returns:
        List[TargetDevice]: filtered list of devices.
    """
    filtered_devices: List = []
    for device in devices:
        if device.device_name == name:
            filtered_devices.append(device)
    return filtered_devices


def filter_devices_with_device_software_version(
    software_version: str, devices: List[TargetDevice]
) -> List[TargetDevice]:
    """Filter the devices with given software version.

    Args:
        software_version (DeviceName): The software version of device.
        devices (List[TargetDevice]): The list of devices
    Returns:
        List[TargetDevice]: filtered list of devices.
    """
    filtered_devices: List = []
    for device in devices:
        if device.software_version == software_version:
            filtered_devices.append(device)
    return filtered_devices


def filter_devices_with_hardware_type(hardware_type: str, devices: List[TargetDevice]) -> List[TargetDevice]:
    """Filter the devices with given hardware type.

    Args:
        software_version (DeviceName): The hardware type of device.
        devices (List[TargetDevice]): The list of devices
    Returns:
        List[TargetDevice]: filtered list of devices.
    """
    filtered_devices: List = []
    for device in devices:
        if device.hardware_type == hardware_type:
            filtered_devices.append(device)
    return filtered_devices



import os, configparser
from pathlib import Path
from enum import Enum
from typing import Literal
from loguru import logger

BASE_DIR = Path(__file__).resolve().parent
config = configparser.ConfigParser()
DEPLOYMENT_MODE = os.getenv("DEPLOYMENT_MODE", "prod")
logger.info(f"Read {DEPLOYMENT_MODE} config")
config.read(f"{BASE_DIR}/configs/config.ini")

class EnvironmentType(str, Enum):
    PROD = "prod"
    LOCAL = "dev"

    @classmethod
    def create_literal(cls):
        return Literal[
            "prod",
            "local"
        ]

class EndPoint(str, Enum):
    GENRAL = "general"
    COMPRESSOR = "compressor"
    LAUNCHER = "launcher"
    @classmethod
    def create_literal(cls):
        return Literal[
            "general",
            "compressor",
            "laucher"
        ]
    
class EndPointProperty(str, Enum):
    HOST = "HOST"
    PORT = "PORT"
    URI_PREFIX = "URI_PREFIX"

    @classmethod
    def create_literal(cls):
        return Literal[
            "HOST",
            "PORT",
            "URI_PREFIX"
        ]
    

class Config:
    ENVIRONMENT_TYPE: EnvironmentType = EnvironmentType.PROD
    CONFIG_SESSION: str = f"{EndPoint.GENRAL}.{ENVIRONMENT_TYPE}"
    HOST: str = config[CONFIG_SESSION][EndPointProperty.HOST]
    PORT: int = int(config[CONFIG_SESSION][EndPointProperty.PORT])
    URI_PREFIX: str = config[CONFIG_SESSION][EndPointProperty.URI_PREFIX]

    def __init__(self, endpoint: EndPoint = EndPoint.GENRAL):
        self.ENVIRONMENT_TYPE = EnvironmentType(DEPLOYMENT_MODE.lower())
        self.CONFIG_SESSION = f"{endpoint}.{self.ENVIRONMENT_TYPE}"
        self.HOST = config[self.CONFIG_SESSION][EndPointProperty.HOST]
        self.PORT = int(config[self.CONFIG_SESSION][EndPointProperty.PORT])
        self.URI_PREFIX = config[self.CONFIG_SESSION][EndPointProperty.URI_PREFIX]




from os.path import basename

from netspresso import __version__
from netspresso.utils.system import ENV_STR


def get_headers(access_token=None, json_type=False):
    headers = {
        "User-Agent": f"NetsPresso Python Package v{__version__} ({ENV_STR})",
    }
    if access_token:
        headers["Authorization"] = f"Bearer {access_token}"
    if json_type:
        headers["Content-Type"] = "application/json"
    return headers


def get_files(file_path):
    return [("file", (basename(file_path), open(file_path, "rb"), "application/octet-stream"))]



from datetime import datetime
import jwt
import pytz

def check_jwt_exp(access_token):
    payload = jwt.decode(access_token, options={"verify_signature": False})
    return datetime.now(pytz.utc).timestamp() <= payload["exp"]



from enum import Enum
from typing import Literal


class Task(str, Enum):
    IMAGE_CLASSIFICATION = "image_classification"
    OBJECT_DETECTION = "object_detection"
    IMAGE_SEGMENTATION = "image_segmentation"
    SEMANTIC_SEGMENTATION = "semantic_segmentation"
    INSTANCE_SEGMENTATION = "instance_segmentation"
    PANOPTIC_SEGMENTATION = "panoptic_segmentation"
    OTHER = "other"

    @classmethod
    def create_literal(cls):
        return Literal[
            "image_classification",
            "object_detection",
            "image_segmentation",
            "semantic_segmentation",
            "instance_segmentation",
            "panoptic_segmentation",
            "other",
        ]


class Framework(str, Enum):
    TENSORFLOW_KERAS = "tensorflow_keras"
    PYTORCH = "pytorch"
    ONNX = "onnx"

    @classmethod
    def create_literal(cls):
        return Literal["tensorflow_keras", "pytorch", "onnx"]


class Extension(str, Enum):
    H5 = "h5"
    ZIP = "zip"
    PT = "pt"
    ONNX = "onnx"

    @classmethod
    def create_literal(cls):
        return Literal["h5", "zip", "pt", "onnx"]


class CompressionMethod(str, Enum):
    PR_L2 = "PR_L2"
    PR_GM = "PR_GM"
    PR_NN = "PR_NN"
    PR_ID = "PR_ID"
    FD_TK = "FD_TK"
    FD_CP = "FD_CP"
    FD_SVD = "FD_SVD"

    @classmethod
    def create_literal(cls):
        return Literal["PR_L2", "PR_GM", "PR_NN", "PR_ID", "FD_TK", "FD_CP", "FD_SVD"]


class RecommendationMethod(str, Enum):
    SLAMP = "slamp"
    VBMF = "vbmf"

    @classmethod
    def create_literal(cls):
        return Literal["slamp", "vbmf"]


class OriginFrom(str, Enum):
    CUSTOM = "custom"
    NPMS = "npms"

    @classmethod
    def create_literal(cls):
        return Literal["custom", "npms"]


class Policy(str, Enum):
    SUM = "sum"
    AVERAGE = "average"
    BACKWARD = "backward"

    @classmethod
    def create_literal(cls):
        return Literal["sum", "average", "backward"]


class GroupPolicy(str, Enum):
    SUM = "sum"
    AVERAGE = "average"
    COUNT = "count"
    NONE = "none"

    @classmethod
    def create_literal(cls):
        return Literal["sum", "average", "count", "none"]


class LayerNorm(str, Enum):
    NONE = "none"
    STANDARD_SCORE = "standard_score"
    TSS_NORM = "tss_norm"
    LINEAR_SCALING = "linear_scaling"
    SOFTMAX_NORM = "softmax_norm"

    @classmethod
    def create_literal(cls):
        return Literal["none", "standard_score", "tss_norm", "linear_scaling", "softmax_norm"]


task_literal = Task.create_literal()
framework_literal = Framework.create_literal()
extension_literal = Extension.create_literal()
compression_literal = CompressionMethod.create_literal()
recommendation_literal = RecommendationMethod.create_literal()
originfrom_literal = OriginFrom.create_literal()
policy_literal = Policy.create_literal()
grouppolicy_literal = GroupPolicy.create_literal()
layernorm_literal = LayerNorm.create_literal()



import json
import requests

from netspresso.schemas.auth import (
    CreditResponse,
    LoginResponse,
    UserResponse,
    RefreshTokenResponse,
)
from netspresso.compressor.client.schemas.compression import (
    CompressionResponse,
    GetAvailableLayersReponse,
    RecommendationResponse,
)
from netspresso.compressor.client.schemas.model import UploadModelRequest, ModelResponse, GetDownloadLinkResponse
from netspresso.client.utils.common import get_files, get_headers
from netspresso.compressor.client.utils.enum import (
    Task,
    Framework,
    Extension,
    OriginFrom,
    CompressionMethod,
    RecommendationMethod,
    Policy,
    LayerNorm,
    GroupPolicy,
)  # noqa
from netspresso.client.config import Config, EndPoint

class ModelCompressorAPIClient:
    def __init__(self):
        self.config = Config(EndPoint.COMPRESSOR)
        self.host = self.config.HOST
        self.port = self.config.PORT
        self.prefix = self.config.URI_PREFIX
        self.url = f"{self.host}:{self.port}{self.prefix}"

    def upload_model(self, data: UploadModelRequest, access_token) -> ModelResponse:
        url = f"{self.url}/models"
        files = get_files(data.file_path)
        response = requests.post(url, data=data.dict(), files=files, headers=get_headers(access_token))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            return ModelResponse(**response_body)
        else:
            raise Exception(response_body["detail"])

    def get_parent_models(self, is_simple, access_token):
        url = f"{self.url}/models/parents?is_simple={is_simple}"
        response = requests.get(url, headers=get_headers(access_token))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            return [ModelResponse(**r) for r in response_body]
        else:
            raise Exception(response_body["detail"])

    def get_children_models(self, model_id, access_token):
        url = f"{self.url}/models/{model_id}/children"
        response = requests.get(url, headers=get_headers(access_token))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            return [ModelResponse(**r) for r in response_body]
        else:
            raise Exception(response_body["detail"])

    def get_model_info(self, model_id, access_token) -> ModelResponse:
        url = f"{self.url}/models/{model_id}"
        response = requests.get(url, headers=get_headers(access_token))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            return ModelResponse(**response_body)
        else:
            raise Exception(response_body["detail"])

    def get_download_model_link(self, model_id, access_token) -> GetDownloadLinkResponse:
        url = f"{self.url}/models/{model_id}/download"
        response = requests.post(url, headers=get_headers(access_token))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            return GetDownloadLinkResponse(**response_body)
        else:
            raise Exception(response_body["detail"])

    def delete_model(self, model_id, access_token):
        url = f"{self.url}/models/{model_id}"
        response = requests.delete(url, headers=get_headers(access_token))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            return response_body
        else:
            raise Exception(response_body["detail"])

    def get_available_layers(self, data, access_token) -> GetAvailableLayersReponse:
        url = f"{self.url}/models/{data.model_id}/get_available_layers"
        response = requests.post(url, data=data.json(), headers=get_headers(access_token, json_type=True))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            return GetAvailableLayersReponse(**response_body)
        else:
            raise Exception(response_body["detail"])

    def create_compression(self, data, access_token) -> CompressionResponse:
        url = f"{self.url}/compressions"
        response = requests.post(url, data=data.json(), headers=get_headers(access_token, json_type=True))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            return CompressionResponse(**response_body)
        else:
            raise Exception(response_body["detail"])

    def get_recommendation(self, data, access_token) -> RecommendationResponse:
        url = f"{self.url}/models/{data.model_id}/recommendation"
        response = requests.post(url, data=data.json(), headers=get_headers(access_token, json_type=True))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            response_body = {"recommended_layers": response_body}
            return RecommendationResponse(**response_body)
        else:
            raise Exception(response_body["detail"])

    def compress_model(self, data, access_token):
        url = f"{self.url}/compressions/{data.compression_id}"
        response = requests.put(url, data=data.json(), headers=get_headers(access_token, json_type=True))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            return CompressionResponse(**response_body)
        else:
            raise Exception(response_body["detail"])

    def auto_compression(self, data, access_token):
        url = f"{self.url}/models/{data.model_id}/auto_compress"
        response = requests.post(url, data=data.json(), headers=get_headers(access_token, json_type=True))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            response = ModelResponse(**response_body)
            return response
        else:
            raise Exception(response_body["detail"])

    def get_compression_info(self, compression_id, access_token):
        url = f"{self.url}/compressions/{compression_id}"

        response = requests.get(url, headers=get_headers(access_token))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            return CompressionResponse(**response_body)
        else:
            raise Exception(response_body["detail"])

    def upload_dataset(self, data, access_token):
        url = f"{self.url}/models/{data.model_id}/datasets"
        files = get_files(data.file_path)
        response = requests.post(url, files=files, headers=get_headers(access_token))
        response_body = json.loads(response.text)

        if response.status_code == 200:
            return response_body
        else:
            raise Exception(response_body["detail"])



from typing import Dict, List, Union

from urllib import request
from loguru import logger

from netspresso.compressor.client import (
    ModelCompressorAPIClient,
    Task,
    Framework,
    Extension,
    OriginFrom,
    CompressionMethod,
    RecommendationMethod,
    Policy,
    LayerNorm,
    GroupPolicy,
)
from netspresso.compressor.client.schemas.model import UploadModelRequest
from netspresso.compressor.client.schemas.compression import (
    AutoCompressionRequest,
    CompressionRequest,
    GetAvailableLayersRequest,
    CreateCompressionRequest,
    RecommendationRequest,
    UploadDatasetRequest,
    AvailableLayer,
    Options,
)
from netspresso.compressor.core.model import CompressedModel, Model, ModelCollection, ModelFactory
from netspresso.compressor.core.compression import CompressionInfo
from netspresso.client import BaseClient, validate_token


class ModelCompressor(BaseClient):
    def __init__(self, email=None, password=None, user_session=None):
        """Initialize the Model Compressor.

        Args:
            email (str): The email address for a user account.
            password (str): The password for a user account.
            user_session (SessionClient): The SessionClient object.

        Available constructors: 
            ModelCompressor(email='USER_EMAIL',password='PASSWORD')
            ModelCompressor(user_session=SessionClient(email='USER_EMAIL',password='PASSWORD')
        """
        super().__init__(email=email, password=password, user_session=user_session)
        self.client = ModelCompressorAPIClient()
        self.model_factory = ModelFactory()

    @validate_token
    def upload_model(
        self, model_name: str, task: Task, framework: Framework, file_path: str, input_shapes: List[Dict[str, int]] = []
    ) -> Model:
        """Upload a model for compression.

        Args:
            model_name (str): The name of the model.
            task (Task): The task of the model.
            framework (Framework): The framework of the model.
            file_path (str): The file path where the model is located.
            input_shapes (List[Dict[str, int]], optional): Input shapes of the model. Defaults to [].

        Raises:
            e: If an error occurs while uploading the model.

        Returns:
            Model: Uploaded model object.
        """

        try:
            logger.info("Uploading Model...")
            data = UploadModelRequest(
                model_name=model_name,
                task=task,
                framework=framework,
                file_path=file_path,
                input_layers=input_shapes,
            )
            model_info = self.client.upload_model(data=data, access_token=self.user_session.access_token)
            model = self.model_factory.create_model(model_info=model_info)

            logger.info(f"Upload model successfully. Model ID: {model.model_id}")

            return model

        except Exception as e:
            logger.error(f"Upload model failed. Error: {e}")
            raise e

    @validate_token
    def get_models(self) -> List[ModelCollection]:
        """Get the list of uploaded & compressed models.

        Raises:
            e: If an error occurs while getting the model list.

        Returns:
            List[ModelCollection]: The list of uploaded & compressed models.
        """

        try:
            logger.info("Getting model list...")
            models = []
            parent_models = self.client.get_parent_models(is_simple=True, access_token=self.user_session.access_token)

            for parent_model_info in parent_models:
                if parent_model_info.origin_from == "custom":
                    children_models = self.client.get_children_models(
                        model_id=parent_model_info.model_id, access_token=self.user_session.access_token
                    )
                    model_collection = self.model_factory.create_model_collection(
                        model_info=parent_model_info, children_models=children_models
                    )
                    models.append(model_collection)
            logger.info("Get model list successfully.")

            return models

        except Exception as e:
            logger.error(f"Get model list failed. Error: {e}")
            raise e

    @validate_token
    def get_uploaded_models(self) -> List[Model]:
        """Get the list of uploaded models.

        Raises:
            e: If an error occurs while getting the uploaded model list.

        Returns:
            List[Model]: The list of uploaded models.
        """

        try:
            logger.info("Getting uploaded model list...")
            parent_models = self.client.get_parent_models(is_simple=True, access_token=self.user_session.access_token)
            uploaded_models = [
                self.model_factory.create_model(model_info=parent_model_info)
                for parent_model_info in parent_models
                if parent_model_info.origin_from == "custom"
            ]
            logger.info("Get uploaded model list successfully.")

            return uploaded_models

        except Exception as e:
            logger.error(f"Get uploaded model list failed. Error: {e}")
            raise e

    @validate_token
    def get_compressed_models(self, model_id: str) -> List[CompressedModel]:
        """Get the list of compressed models for a given model ID.

        Args:
            model_id (str): The ID of the model.

        Raises:
            e: If an error occurs while getting the compressed model list.

        Returns:
            List[CompressedModel]: The list of compressed models for a given model ID.
        """

        try:
            logger.info("Getting compressed model list...")
            children_models = self.client.get_children_models(model_id=model_id, access_token=self.user_session.access_token)
            compressed_models = [
                self.model_factory.create_compressed_model(model_info=children_model_info)
                for children_model_info in children_models
            ]
            logger.info("Get compressed model list successfully.")

            return compressed_models

        except Exception as e:
            logger.error(f"Get compressed model list failed. Error: {e}")
            raise e

    @validate_token
    def get_model(self, model_id: str) -> Union[Model, CompressedModel]:
        """Get the model for a given model ID.

        Args:
            model_id (str): The ID of the model.

        Raises:
            e: If an error occurs while getting the model.

        Returns:
            Union[Model, CompressedModel]: The retrieved model. If the model is compressed,
            `CompressedModel` will be returned. Otherwise, `Model` will be returned.
        """

        try:
            logger.info("Getting model...")
            model_info = self.client.get_model_info(model_id=model_id, access_token=self.user_session.access_token)
            if model_info.status.is_compressed:
                model = self.model_factory.create_compressed_model(model_info=model_info)
            else:
                model = self.model_factory.create_model(model_info=model_info)
            logger.info("Get model successfully.")

            return model

        except Exception as e:
            logger.error(f"Get model failed. Error: {e}")
            raise e

    @validate_token
    def download_model(self, model_id: str, local_path: str) -> None:
        """Download the model for a given model ID to the local path.

        Args:
            model_id (str): The ID of the model.
            local_path (str): The local path to save the downloaded model.

        Raises:
            e: If an error occurs while downloading the model.
        """

        try:
            logger.info("Downloading model...")
            download_link = self.client.get_download_model_link(model_id=model_id, access_token=self.user_session.access_token)
            request.urlretrieve(download_link.url, local_path)
            logger.info(f"Download model successfully. Local Path: {local_path}")

        except Exception as e:
            logger.error(f"Download model failed. Error: {e}")
            raise e

    @validate_token
    def delete_model(self, model_id: str, recursive: bool = False) -> None:
        """Delete the model for a given model ID.

        Args:
            model_id (str): The ID of the model.
            recursive (bool, optional): Whether to also delete the compressed model for that model. Defaults to False.

        Raises:
            e: If an error occurs while deleting the model.
        """

        try:
            logger.info("Deleting model...")
            children_models = self.client.get_children_models(model_id=model_id, access_token=self.user_session.access_token)
            if len(children_models) != 0:
                if not recursive:
                    logger.warning(
                        "Deleting the model will also delete its compressed models. To proceed with the deletion, set the `recursive` parameter to True."
                    )
                else:
                    logger.info("The compressed model for that model will also be deleted.")
                    self.client.delete_model(model_id=model_id, access_token=self.user_session.access_token)
                    logger.info("Delete model successfully.")
            else:
                logger.info("The model will be deleted.")
                self.client.delete_model(model_id=model_id, access_token=self.user_session.access_token)
                logger.info("Delete model successfully.")

        except Exception as e:
            logger.error(f"Delete model failed. Error: {e}")
            raise e

    @validate_token
    def select_compression_method(
        self, model_id: str, compression_method: CompressionMethod, options: Options = Options()
    ) -> CompressionInfo:
        """Select a compression method for a model.

        Args:
            model_id (str): The ID of the model.
            compression_method (CompressionMethod): The selected compression method.
            options(Options, optional): The options for pruning method.

        Raises:
            e: If an error occurs while selecting the compression method.

        Returns:
            CompressionInfo: The compression information for the selected compression method.
        """
        try:
            model = self.get_model(model_id)

            logger.info("Selecting compression method...")
            if model.framework == Framework.PYTORCH and compression_method == CompressionMethod.PR_NN:
                raise Exception("The Nuclear Norm is only supported in the TensorFlow-Keras framework.")

            data = GetAvailableLayersRequest(model_id=model.model_id, compression_method=compression_method, options=options)
            response = self.client.get_available_layers(data=data, access_token=self.user_session.access_token)
            compression_info = CompressionInfo(
                original_model_id=model.model_id, compression_method=compression_method, options=options.dict()
            )
            compression_info.set_available_layers(response.available_layers)
            logger.info("Select compression method successfully.")

            return compression_info

        except Exception as e:
            logger.error(f"Select compression method failed. Error: {e}")
            raise e

    @validate_token
    def get_compression(self, compression_id: str) -> CompressionInfo:
        """Get information about a compression.

        Args:
            compression_id (str): The ID of the compression.

        Raises:
            e: If an error occurs while getting the compression information.

        Returns:
            CompressionInfo: The information about the compression.
        """
        try:
            logger.info("Getting compression...")
            compression_info = self.client.get_compression_info(
                compression_id=compression_id, access_token=self.user_session.access_token
            )
            compression_info = CompressionInfo(
                compressed_model_id=compression_info.new_model_id,
                compression_id=compression_info.compression_id,
                compression_method=compression_info.compression_method,
            )
            compression_info.set_available_layers(compression_info.compression_method)
            logger.info("Get compression successfully.")

            return compression_info

        except Exception as e:
            logger.error(f"Get compression failed. Error: {e}")
            raise e

    @validate_token
    def __upload_dataset(self, model_id: str, dataset_path: str) -> None:
        """Upload a dataset for nuclear norm compression method.

        Args:
            model_id (str): The ID of the model.
            dataset_path (str): The file path where the dataset is located.

        Raises:
            e: If an error occurs while uploading the dataset.
        """
        try:
            logger.info(f"Uploading dataset...")
            data = UploadDatasetRequest(model_id=model_id, file_path=dataset_path)
            self.client.upload_dataset(data=data, access_token=self.user_session.access_token)
            logger.info(f"Upload dataset successfully.")

        except Exception as e:
            logger.error(f"Upload dataset failed. Error: {e}")
            raise e

    @validate_token
    def compress_model(
        self, compression: CompressionInfo, model_name: str, output_path: str, dataset_path: str = None
    ) -> CompressedModel:
        """Compress a model using the provided compression information.

        Args:
            compression (CompressionInfo): The information about the compression.
            model_name (str): The name of the compressed model.
            output_path (str): The local path to save the compressed model.
            dataset_path (str, optional): The path of the dataset used for nuclear norm compression method. Default is None.

        Raises:
            e: If an error occurs while compressing the model.

        Returns:
            CompressedModel: The compressed model.
        """
        try:
            logger.info("Compressing model...")
            data = CreateCompressionRequest(
                model_id=compression.original_model_id,
                model_name=model_name,
                compression_method=compression.compression_method,
                options=compression.options,
            )
            compression_info = self.client.create_compression(data=data, access_token=self.user_session.access_token)

            if dataset_path and compression.compression_method == CompressionMethod.PR_NN:
                self.__upload_dataset(model_id=compression.original_model_id, dataset_path=dataset_path)

            for available_layers in compression.available_layers:
                if available_layers.values != [""]:
                    available_layers.use = True

            all_layers_false = all(available_layer.values == [""] for available_layer in compression.available_layers)
            if all_layers_false:
                raise Exception(
                    f"The available_layer.values all empty. please put in the available_layer.values to compress."
                )

            available_layers = [
                AvailableLayer(name=layer.name, values=layer.values, channels=layer.channels, use=layer.use)
                for layer in compression.available_layers
            ]

            data = CompressionRequest(
                compression_id=compression_info.compression_id,
                compression_method=compression.compression_method,
                layers=available_layers,
                compressed_model_id=compression_info.new_model_id,
                options=compression.options,
            )
            self.client.compress_model(data=data, access_token=self.user_session.access_token)
            self.download_model(model_id=compression_info.new_model_id, local_path=output_path)
            compressed_model = self.get_model(model_id=compression_info.new_model_id)
            logger.info(f"Compress model successfully. Compressed Model ID: {compressed_model.model_id}")
            logger.info("50 credits have been consumed.")

            return compressed_model

        except Exception as e:
            logger.error(f"Compress model failed. Error: {e}")
            raise e

    @validate_token
    def recommendation_compression(
        self,
        model_id: str,
        model_name: str,
        compression_method: CompressionMethod,
        recommendation_method: RecommendationMethod,
        recommendation_ratio: float,
        output_path: str,
        options: Options = Options(),
        dataset_path: str = None,
    ) -> CompressedModel:
        """Compress a recommendation-based model using the given compression and recommendation methods.

        Args:
            model_id (str): The ID of the model.
            model_name (str): The name of the compressed model.
            compression_method (CompressionMethod): The selected compression method.
            recommendation_method (RecommendationMethod): The selected recommendation method.
            recommendation_ratio (float): The compression ratio recommended by the recommendation method.
            output_path (str): The local path to save the compressed model.
            options(Options, optional): The options for pruning method.
            dataset_path (str, optional): The path of the dataset used for nuclear norm compression method. Default is None.

        Raises:
            e: If an error occurs while performing recommendation compression.

        Returns:
            CompressedModel: The compressed model.
        """

        try:
            model = self.get_model(model_id)

            logger.info("Compressing recommendation-based model...")

            if model.framework == Framework.PYTORCH and compression_method == CompressionMethod.PR_NN:
                raise Exception("The Nuclear Norm is only supported in the TensorFlow-Keras framework.")

            if compression_method in [CompressionMethod.PR_ID, CompressionMethod.FD_CP]:
                raise Exception(
                    f"The {compression_method} compression method you choose doesn't provide a recommendation."
                )

            if (
                compression_method in [CompressionMethod.PR_L2, CompressionMethod.PR_GM, CompressionMethod.PR_NN]
                and recommendation_method != RecommendationMethod.SLAMP
            ):
                raise Exception(
                    f"The {compression_method} compression method is only available the SLAMP recommendation method."
                )

            if (
                compression_method in [CompressionMethod.FD_TK, CompressionMethod.FD_SVD]
                and recommendation_method != RecommendationMethod.VBMF
            ):
                raise Exception(
                    f"The {compression_method} compression method is only available the VBMF recommendation method."
                )

            data = CreateCompressionRequest(
                model_id=model.model_id, model_name=model_name, compression_method=compression_method, options=options.dict()
            )
            compression_info = self.client.create_compression(data=data, access_token=self.user_session.access_token)

            if dataset_path and compression_method == CompressionMethod.PR_NN:
                self.__upload_dataset(model_id=model.model_id, dataset_path=dataset_path)

            data = RecommendationRequest(
                model_id=model.model_id,
                compression_id=compression_info.compression_id,
                recommendation_method=recommendation_method,
                recommendation_ratio=recommendation_ratio,
                options=options.dict(),
            )
            recommendation_result = self.client.get_recommendation(data=data, access_token=self.user_session.access_token)

            for recommended_layer in recommendation_result.recommended_layers:
                for available_layer in compression_info.available_layers:
                    # Find the matching available_layer by name
                    if available_layer.name == recommended_layer.name:
                        available_layer.use = True
                        available_layer.values = recommended_layer.values

            data = CompressionRequest(
                compression_id=compression_info.compression_id,
                compression_method=compression_method,
                layers=compression_info.available_layers,
                compressed_model_id=compression_info.new_model_id,
                options=options.dict(),
            )
            self.client.compress_model(data=data, access_token=self.user_session.access_token)
            self.download_model(model_id=compression_info.new_model_id, local_path=output_path)
            compressed_model = self.get_model(model_id=compression_info.new_model_id)
            logger.info(f"Recommendation compression successfully. Compressed Model ID: {compressed_model.model_id}")
            logger.info("50 credits have been consumed.")

            return compressed_model

        except Exception as e:
            logger.error(f"Recommendation compression failed. Error: {e}")
            raise e

    @validate_token
    def automatic_compression(
        self, model_id: str, model_name: str, output_path: str, compression_ratio: float = 0.5
    ) -> CompressedModel:
        """Compress a model automatically based on the given compression ratio.

        Args:
            model_id (str): The ID of the model.
            model_name (str): The name of the compressed model.
            output_path (str): The local path to save the compressed model.
            compression_ratio (float): The compression ratio for automatic compression. Defaults to 0.5.

        Raises:
            e: If an error occurs while performing automatic compression.

        Returns:
            CompressedModel: The compressed model.
        """

        try:
            logger.info("Compressing automatic-based model...")
            data = AutoCompressionRequest(
                model_id=model_id,
                model_name=model_name,
                recommendation_ratio=compression_ratio,
                save_path=output_path,
            )
            model_info = self.client.auto_compression(data=data, access_token=self.user_session.access_token)
            self.download_model(model_id=model_info.model_id, local_path=output_path)
            compressed_model = self.model_factory.create_compressed_model(model_info=model_info)
            logger.info(f"Automatic compression successfully. Compressed Model ID: {compressed_model.model_id}")
            logger.info("25 credits have been consumed.")

            return compressed_model

        except Exception as e:
            logger.error(f"Automatic compression failed. Error: {e}")
            raise e



import json

from typing import List, Optional, Union
from pydantic import BaseModel, Field, validator

from netspresso.launcher.schemas import DataType, DeviceName, TaskStatus, ModelFramework, HardwareType


class InputShape(BaseModel):
    """Represents the shape of an input tensor.

    Attributes:
        batch (int): The batch size of the input tensor.
        channel (int): The number of channels in the input tensor.
        dimension (str): The dimensions of the input tensor. ex: height = 416, width = 640, [416, 640].
    """

    batch: Optional[int] = Field(default=1)
    channel: int
    input_size: str

    @validator("batch", always=True, allow_reuse=True)
    def set_batch(cls, v, values, **kwargs):
        return v if v is not None else 1


class TargetDevice(BaseModel):
    """Represents the target device.

    Attributes:
        display_name (str): devices's display name.
        display_brand_name (str): the manufacturer name of the device.
        device_name (str): internal devcie name (using as an internal identifier)
        software_version (Optional[str]): software version of the device (ex: jetpack 5.0.1)
        software_version_display_name (Optional[str]): display name of the devcie's software version
    """

    display_name: str = Field(default=None)
    display_brand_name: str = Field(default=None)
    device_name: DeviceName = Field(default=None)
    software_version: Optional[str] = Field(default=None)
    software_version_display_name: Optional[str] = Field(default=None)
    hardware_type: Optional[HardwareType] = Field(default=None)


class Model(BaseModel):
    """Represents the launcher model object.

    Attributes:
        framework (ModelFramework): the framework of the model.
        filename (str): the file name of the model.
        input_shape (InputShape): the input shape of the model.
        data_type (DataType): the data_type of the model.
        available_devices (List[TargetDevice]): avaliable devices for conversion or benchmark.
        model_uuid (str): the launcher model uuid.
        file_size (float): the size of the model.
    """

    framework: ModelFramework = Field(default=None)
    filename: str = Field(default=None)
    input_shape: InputShape = Field(default=None)
    data_type: DataType = Field(default=None)
    available_devices: List[TargetDevice] = Field(default_factory=list)
    model_uuid: str = Field(default=None)
    file_size: float = Field(default=None)


class BaseRequestModel(BaseModel):
    """Represents the launcher base request object.

    Attributes:
        user_uuid (str): the unique user identifier.
        input_model_uuid (str): the uuid of launcher model object.
        software_version (Optional[str]): target device's software version.
    """

    user_uuid: str = Field(default=None)
    input_model_uuid: str = Field(default=None)
    software_version: Optional[str] = Field(default=None)


class ModelBenchmarkRequest(BaseRequestModel):
    """Represents the launcher benchmark request object.

    Attributes:
        target_device (DeviceName): the device name.
        target_framework (Optional[ModelFramework]): the target framework of the model.
        data_type: (Optional[DataType]): data type of the model.
        input_shape: (Optional[InputShape]): input shape of the model.
    """

    target_device: DeviceName = Field(default=None)
    target_framework: Optional[ModelFramework] = Field(default=None)
    data_type: Optional[DataType] = Field(default=None)
    input_shape: Optional[InputShape] = Field(default=None)
    hardware_type: Optional[HardwareType] = Field(default=None)


class ModelConversionRequest(BaseRequestModel):
    """Represents the launcher conversion request object.

    Attributes:
        target_device (DeviceName): the device name.
        target_framework (ModelFramework): the target framework of the model.
        data_type: (DataType): data type of the model.
        input_shape: (InputShape): input shape of the model.
    """

    target_device_name: DeviceName = Field(default=None)
    target_framework: ModelFramework = Field(default=None)
    data_type: DataType = Field(default=DataType.FP16)
    input_shape: InputShape = Field(default=None)

    @validator("input_shape")
    def validate_input_shape(cls, value):
        if value:
            return json.dumps(value.dict())
        else:
            return None


class BaseTaskModel(BaseModel):
    """Represents the launcher base task response object.

    Attributes:
        user_uuid (str): the unique user identifier.
        input_model_uuid (str): the uuid of launcher model object.
        status (TaskStatus): current status of the task.
        input_shape: (InputShape): input shape of the model.
        data_type: (DataType): data type of the model.
        software_version (str): target device's software version.
        framework (ModelFramework): the target framework of the model.
    """

    user_uuid: str = Field(default=None)
    input_model_uuid: str = Field(default=None)
    status: TaskStatus = Field(default=None)
    input_shape: InputShape = Field(default=None)
    data_type: DataType = Field(default=DataType.FP16)
    input_shape: InputShape = Field(default=None)
    software_version: str = Field(default=None)
    framework: ModelFramework = Field(default=None)


class BenchmarkTask(BaseTaskModel):
    """Represents the launcher benchmark task response object.

    Attributes:
        benchmark_task_uuid (str): the unique benchamrk task identifier.
        target_device (DeviceName): the uuid of launcher model object.
        file_name (str): the model file name.
        data_type: (Optional[DataType]): data type of the model
        memory_footprint_gpu (Optional[float]): memory usage of the model while inferencing the model.
        memory_footprint_cpu (Optional[float]): cpu usage of the model while inferencing the model.
        latency (Optional[float]): average inference latency of the model.
        ram_size (Optional[float]): RAM size of the target device.
        processor (Optional[str]): processor information of the target device.
        benchmark_result (Optional[dict]): original benchmark result in dictionary.
    """

    benchmark_task_uuid: str = Field(default=None)
    target_device: DeviceName = Field(default=None)
    file_name: str = Field(default=None)
    data_type: Optional[DataType] = Field(default=DataType.FP16)
    memory_footprint_gpu: Optional[float] = Field(default=None)
    memory_footprint_cpu: Optional[float] = Field(default=None)
    latency: Optional[float] = Field(default=None)
    ram_size: Optional[float] = Field(default=None)
    processor: Optional[str] = Field(default=None)
    benchmark_result: Optional[dict] = Field(default=None)


class ConversionTask(BaseTaskModel):
    """Represents the launcher benchmark task response object.

    Attributes:
        convert_task_uuid (str): the unique convert task identifier.
        output_model_uuid (str): the uuid of converted model.
        model_file_name (str): the model file name.
        target_device_name (DeviceName): the device name.
    """

    convert_task_uuid: str = Field(default=None)
    output_model_uuid: str = Field(default=None)
    model_file_name: str = Field(default=None)
    target_device_name: DeviceName = Field(default=None)



from enum import Enum
from typing import Literal


class StrEnumBase(str, Enum):
    def __repr__(self) -> str:
        return str.__repr__(self.value)


class LauncherFunction(StrEnumBase):
    GENERAL = "GENERAL"
    CONVERT = "CONVERT"
    BENCHMARK = "BENCHMARK"

    @classmethod
    def create_literal(cls):
        return Literal[
            "GENERAL",
            "CONVERT",
            "BENCHMARK",
        ]


class DataType(StrEnumBase):
    FP32 = "FP32"
    FP16 = "FP16"
    INT8 = "INT8"
    NONE = ""

    @classmethod
    def create_literal(cls):
        return Literal[
            "FP32",
            "FP16",
            "INT8",
            "",
        ]


class ModelFramework(StrEnumBase):
    ONNX = "onnx"
    TENSORRT = "tensorrt"
    OPENVINO = "openvino"
    TENSORFLOW_LITE = "tensorflow_lite"
    DRPAI = "drpai"
    TENSORFLOW_KERAS = "keras"
    TENSORFLOW = "saved_model"

    @classmethod
    def create_literal(cls):
        return Literal[
            "onnx",
            "tensorrt",
            "openvino",
            "tensorflow_lite",
            "drpai",
            "keras",
            "saved_model",
        ]


class DeviceName(StrEnumBase):
    RASPBERRY_PI_4B = "RaspberryPi4B"
    RASPBERRY_PI_3B_PLUS = "RaspberryPi3BPlus"
    RASPBERRY_PI_ZERO_W = "RaspberryPi-ZeroW"
    RASPBERRY_PI_ZERO_2W = "RaspberryPi-Zero2W"
    RENESAS_RZ_V2L = "rzv2l_avnet"
    RENESAS_RZ_V2M = "rzv2m"
    JETSON_NANO = "Jetson-Nano"
    JETSON_TX2 = "Jetson-Tx2"
    JETSON_XAVIER = "Jetson-Xavier"
    JETSON_NX = "Jetson-Nx"
    JETSON_AGX_ORIN = "Jetson-AGX-Orin"
    AWS_T4 = "AWS-T4"
    Intel_XEON_W_2233 = "Intel-Xeon"
    ALIF_ENSEMBLE_E7_DEVKIT_GEN2 = "Ensemble-E7-DevKit-Gen2"
    RENESAS_RA8D1 = "Renesas-RA8D1"

    @classmethod
    def create_literal(cls):
        return Literal[
            "RaspberryPi4B",
            "RaspberryPi3BPlus",
            "RaspberryPi-ZeroW",
            "RaspberryPi-Zero2W",
            "rzv2l_avnet",
            "rzv2m",
            "Jetson-Nano",
            "Jetson-Tx2",
            "Jetson-Xavier",
            "Jetson-Nx",
            "Jetson-AGX-Orin",
            "AWS-T4",
            "Intel-Xeon",
            "Ensemble-E7-DevKit-Gen2",
            "Renesas-RA8D1",
        ]


JETSON_DEVICES = [
    DeviceName.JETSON_NANO,
    DeviceName.JETSON_TX2,
    DeviceName.JETSON_XAVIER,
    DeviceName.JETSON_NX,
    DeviceName.JETSON_AGX_ORIN,
]
RASPBERRY_PI_DEVICES = [
    DeviceName.RASPBERRY_PI_4B,
    DeviceName.RASPBERRY_PI_3B_PLUS,
    DeviceName.RASPBERRY_PI_ZERO_W,
    DeviceName.RASPBERRY_PI_ZERO_2W,
]
RENESAS_DEVICES = [DeviceName.RENESAS_RZ_V2L, DeviceName.RENESAS_RZ_V2M]
NVIDIA_GRAPHIC_CARDS = [DeviceName.AWS_T4]
INTEL_DEVICES = [DeviceName.Intel_XEON_W_2233]
ONLY_INT8_DEVICES = [DeviceName.ALIF_ENSEMBLE_E7_DEVKIT_GEN2, DeviceName.RENESAS_RA8D1]


class SoftwareVersion(StrEnumBase):
    JETPACK_4_4_1 = "4.4.1-b50"
    JETPACK_4_6 = "4.6-b199"
    JETPACK_5_0_1 = "5.0.1-b118"
    JETPACK_5_0_2 = "5.0.2-b231"

    @classmethod
    def create_literal(cls):
        return Literal[
            "4.4.1-b50",
            "4.6-b199",
            "5.0.1-b118",
            "5.0.2-b231",
        ]


class HardwareType(StrEnumBase):
    HELIUM = "helium"

    @classmethod
    def create_literal(cls):
        return Literal["helium"]


class TaskStatus(StrEnumBase):
    IN_QUEUE = "IN_QUEUE"
    IN_PROGRESS = "IN_PROGRESS"
    FINISHED = "FINISHED"
    ERROR = "ERROR"
    USER_CANCEL = "USER_CANCEL"

    @classmethod
    def create_literal(cls):
        return Literal["IN_QUEUE", "IN_PROGRESS", "FINISHED", "ERROR", "USER_CANCEL"]



import json
from typing import List, Any
from os.path import basename

from pydantic import BaseModel, Field, validator, root_validator, HttpUrl

from netspresso.compressor.client.utils.enum import (
    Framework,
    Extension,
    task_literal,
    framework_literal,
    originfrom_literal,
)


class InputLayer(BaseModel):
    batch: Any = Field(0, description="Input Batch")
    channel: int = Field(0, description="Input Channel")
    dimension: List[int] = Field([0], description="Input Diemension")


class UploadModelRequest(BaseModel):
    model_name: str = Field(..., description="Model Name")
    description: str = Field("", description="Description")
    task: task_literal = Field(..., description="Task")
    framework: framework_literal = Field(..., description="Framework")
    input_layers: List[InputLayer] = Field(None, description="Input Layers")
    file_path: str = Field(..., description="Model Path")
    # metric_unit: str = Field("", description="Metric Unit")
    # metric_value: float = Field(0, description="Metric Value")

    @validator("input_layers")
    def validate_input_layers(cls, value):
        if value:
            input_layers = []

            if len(value) != 1:
                raise Exception("Currently, only single input models are supported.")

            for v in value:
                if any((v.batch == 0, v.channel == 0, v.dimension == [0])):
                    return None
                input_layers.append(v.dict())
            return json.dumps(input_layers)
        else:
            return None

    @root_validator(pre=True, skip_on_failure=True)
    def validate_request_params(cls, values):
        framework = values.get("framework")
        input_layers = values.get("input_layers")
        file_path = values.get("file_path")
        file_extension = basename(file_path).split(".")[1]

        if framework not in [Framework.TENSORFLOW_KERAS, Framework.PYTORCH, Framework.ONNX]:
            raise Exception("Invalid framework. Supported frameworks are TensorFlow/Keras, PyTorch, and ONNX.")

        if framework == Framework.TENSORFLOW_KERAS and not file_extension in [Extension.H5, Extension.ZIP]:
            raise Exception(
                "Invalid file extension or framework. TensorFlow/Keras models should have .h5 or .zip extension."
            )
        elif framework == Framework.PYTORCH and not file_extension == Extension.PT:
            raise Exception("Invalid file extension or framework. PyTorch models should have .pt extension.")
        elif framework == Framework.ONNX and not file_extension == Extension.ONNX:
            raise Exception("Invalid file extension or framework. ONNX models should have .onnx extension.")

        if framework == Framework.PYTORCH and input_layers is None:
            raise Exception("Invalid input shape. Input shape is required for PyTorch models.")

        return values


class Spec(BaseModel):
    input_layers: List[InputLayer] = Field([], description="Input Layers")
    model_size: float = Field(0, description="Model Size")
    flops: float = Field(0, description="FLOPs")
    trainable_parameters: float = Field(0, description="Trainable Parameters")
    non_trainable_parameters: float = Field(0, description="Non Trainable Parameters")
    number_of_layers: int = Field(0, description="Number of Layers")


class Status(BaseModel):
    is_convertible: bool = Field(False, description="Convertible Status")
    is_packageable: bool = Field(False, description="Packageable Status")
    is_compressible: bool = Field(False, description="Compressible Status")
    is_visible: bool = Field(False, description="Visible Status")
    is_compressed: bool = Field(False, description="Compressed Status")
    is_trained: bool = Field(False, description="Trained Status")
    is_downloadable: bool = Field(False, description="Downloadable Status")
    is_retrainable: bool = Field(False, description="Retrainable Status")


class Metric(BaseModel):
    metric_unit: str = Field("", description="Metric Unit")
    metric_value: float = Field(None, description="Metric Value")


class Device(BaseModel):
    name: str = Field("", description="Device Name")
    total_latency: float = Field(0, description="Total Latency of Model")
    performance: List = Field([], description="Metric Value")
    spec: List = Field([], description="Metric Value")
    layers: List = Field([], description="Metric Value")


class ModelResponse(BaseModel):
    model_id: str = Field(..., description="Model ID")
    model_name: str = Field(..., description="Model Name")
    description: str = Field("", description="Description")
    original_model_id: str = Field(..., description="Original Model ID")
    original_compression_id: str = Field("", description="Compression ID")
    task: task_literal = Field(..., description="Task")
    framework: framework_literal = Field(..., description="Framework")
    origin_from: originfrom_literal = Field(..., description="Origin From(Model Source)")
    target_device: str = Field("", description="Target Device")
    metric: Metric = Field(..., description="Metric")
    spec: Spec = Field(..., description="Spec")
    status: Status = Field(..., description="Status")
    devices: List[Device] = Field([], description="Devices")
    # edges: List = Field([], description="Edges")
    # nodes: List = Field([], description="Nodes")


class GetDownloadLinkResponse(BaseModel):
    url: HttpUrl = Field(..., description="Model Path")



import json
import requests

from netspresso.client.utils.common import get_files, get_headers
from netspresso.client import Config, EndPoint, SessionClient
from netspresso.launcher.schemas.model import (
    Model,
    ModelBenchmarkRequest,
    BenchmarkTask,
    ModelConversionRequest,
    ConversionTask,
    InputShape,
)
from netspresso.launcher.schemas import LauncherFunction, ModelFramework, DeviceName, DataType


class LauncherAPIClient:
    def __init__(self, user_sessoin: SessionClient):
        self.config = Config(EndPoint.LAUNCHER)
        self.host = self.config.HOST
        self.port = self.config.PORT
        self.prefix = self.config.URI_PREFIX
        self.user_session = user_sessoin
        self.url = f"{self.host}:{self.port}{self.prefix}"

    def upload_model(self, model_file_path: str, target_function: LauncherFunction) -> Model:
        """Upload a model for launcher.

        Args:
            model_file_path (str): The file path of the model.
            target_function (LauncherFunction): speicfy the function of the launcher. Upload for converter or benchmarker

        Raises:
            e: If an error occurs while uploading the model.

        Returns:
            Model: Uploaded launcher model object.
        """
        url = f"{self.url}/{target_function.value.lower()}/upload_model"
        files = get_files(model_file_path)
        # files = {"file": open(model_file_path, "rb")}
        response = requests.post(url, files=files, headers=get_headers(self.user_session.access_token))
        response_body = json.loads(response.text)
        if response.status_code < 300:
            return Model(**response_body)
        else:
            raise Exception(response_body["detail"])

    def convert_model(
        self,
        model_uuid: str,
        input_shape: InputShape,
        target_framework: ModelFramework,
        target_device: DeviceName,
        data_type: DataType,
        software_version: str,
    ) -> ConversionTask:
        """Convert a model into the type the specific framework.

        Args:
            model_uuid (str): The uuid of the launcher model.
            input_shape (InputShape) : target input shape to convert. (ex: dynamic batch to static batch)
            target_framework (ModelFramework): the target framework name.
            target_device (DeviceName): target device.
            data_type (DataType): data type of the model.
            software_version (str): target device's software version.

        Raises:
            e: If an error occurs while converting the model.

        Returns:
            ConversionTask: model conversion task object.
        """

        url = f"{self.url}/convert"
        request_data = ModelConversionRequest(
            user_uuid=self.user_session.user_id,
            target_device_name=target_device,
            input_model_uuid=model_uuid,
            target_framework=target_framework,
            data_type=data_type,
            input_shape=input_shape,
            software_version=software_version,
        )
        if software_version is not None:
            request_data.software_version = software_version

        response = requests.post(url, data=request_data.dict(), headers=get_headers(self.user_session.access_token))
        response_body = json.loads(response.text)
        if response.status_code < 300:
            return ConversionTask(**response_body)
        else:
            raise Exception(response_body["detail"])

    def get_conversion_task(self, conversion_task_uuid: str) -> ConversionTask:
        """Get the conversion task information with given conversion task uuid.

        Args:
            conversion_task_uuid (str): The uuid of the conversion task.

        Raises:
            e: If an error occurs while getting the conversion task information.

        Returns:
            ConversionTask: model conversion task object.
        """

        url = f"{self.url}/convert/{conversion_task_uuid}"
        response = requests.get(url, headers=get_headers(self.user_session.access_token))
        response_body = json.loads(response.text)
        if response.status_code < 300:
            return ConversionTask(**response_body)
        else:
            raise Exception(response_body["detail"])

    def get_converted_model(self, conversion_task_uuid: str):
        """Download the converted model with given conversion task or conversion task uuid.

        Args:
            conversion_task (ConversionTask | str): Launcher Model Object or the uuid of the conversion task.

        Raises:
            e: If an error occurs while getting the conversion task information.

        Returns:
            ConversionTask: model conversion task object.
        """
        url = f"{self.url}/convert/{conversion_task_uuid}/download"
        response = requests.get(url, headers=get_headers(self.user_session.access_token))
        response_body = json.loads(response.text)
        if response.status_code < 300:
            return response_body
        else:
            raise Exception(response_body["detail"])

    def benchmark_model(
        self,
        model_uuid: str,
        target_device: DeviceName,
        data_type: DataType,
        software_version: str = None,
        hardware_type: str = None,
    ) -> BenchmarkTask:
        """Benchmark given model on the specified device.

        Args:
            model_uuid (str): The conversion task object, Launcher Model Object or the uuid of the model.
            target_device (TargetDevice): target device.
            data_type (DataType): data type of the model.
            wait_until_done (bool): if true, wait for the conversion result before returning the function. If false, request the conversion and return the function immediately.

        Raises:
            e: If an error occurs while benchmarking of the model.

        Returns:
            BenchmarkTask: model benchmark task object.
        """
        url = f"{self.url}/benchmark"
        request_data = ModelBenchmarkRequest(
            user_uuid=self.user_session.user_id,
            input_model_uuid=model_uuid,
            target_device=target_device,
            data_type=data_type,
            software_version=software_version,
            hardware_type=hardware_type,
        )
        response = requests.post(url, json=request_data.dict(), headers=get_headers(self.user_session.access_token))
        response_body = json.loads(response.text)
        if response.status_code < 300:
            return BenchmarkTask(**response_body)
        else:
            raise Exception(response_body["detail"])

    def get_benchmark(self, benchmark_task_uuid: str) -> BenchmarkTask:
        """Get the benchmark task information with given benchmark task uuid.

        Args:
            benchmark_task_uuid (str): the uuid of the benchmark task.

        Raises:
            e: If an error occurs while getting the benchmark task information.

        Returns:
            BenchmarkTask: model benchmark task object.
        """
        url = f"{self.url}/benchmark/{benchmark_task_uuid}"
        response = requests.get(url, headers=get_headers(self.user_session.access_token))
        response_body = json.loads(response.text)
        if response.status_code < 300:
            return BenchmarkTask(**response_body)
        else:
            raise Exception(response_body["detail"])
